@@ -21,6 +21,7 @@ from copy import deepcopy
 from functools import wraps
 from inspect import isclass
 
+__version__ = '2.2.1'
 __all__ = [
     'BareField',
     'BigIntegerField',
@@ -370,10 +371,30 @@ class Func(Node):
     def __init__(self, name, *arguments):
         self.name = name
         self.arguments = arguments
+        self._coerce = True
         super(Func, self).__init__()
 
+    @returns_clone
+    def coerce(self, coerce=True):
+        self._coerce = coerce
+
     def clone_base(self):
-        return Func(self.name, *self.arguments)
+        res = Func(self.name, *self.arguments)
+        res._coerce = self._coerce
+        return res
+
+    def over(self, partition_by=None, order_by=None):
+        # Basic window function support.
+        over_clauses = []
+        if partition_by:
+            over_clauses.append(Clause(
+                SQL('PARTITION BY'),
+                *partition_by))
+        if order_by:
+            over_clauses.append(Clause(
+                SQL('ORDER BY'),
+                *order_by))
+        return Clause(self, SQL('OVER'), EnclosedClause(Clause(*over_clauses)))
 
     def __getattr__(self, attr):
         def dec(*args, **kwargs):
@@ -399,15 +420,14 @@ class Clause(Node):
         clone.parens = self.parens
         return clone
 
-class EnclosedClause(Clause):
-    """One or more Node objects joined by commas and enclosed in parens."""
-    glue = ', '
-    parens = True
-
 class CommaClause(Clause):
     """One or more Node objects joined by commas, no parens."""
     glue = ', '
 
+class EnclosedClause(CommaClause):
+    """One or more Node objects joined by commas and enclosed in parens."""
+    parens = True
+
 class Entity(Node):
     """A quoted-name or entity, e.g. "table"."column"."""
     def __init__(self, *path):
@@ -520,10 +540,6 @@ class Field(Node):
         field_type = self.get_db_field()
         return self.get_database().compiler().get_column_type(field_type)
 
-    def field_attributes(self):
-        """Arbitrary default attributes, e.g. "max_length" or "precision"."""
-        return {}
-
     def get_db_field(self):
         return self.db_field
 
@@ -918,12 +934,6 @@ class CompositeKey(object):
     def __set__(self, instance, value):
         pass
 
-"""
-TODO:
-
-Join and .join() are very model-centric, but *should* be able to accept an
-arbitrary single-source (table or select).
-"""
 
 class QueryCompiler(object):
     # Mapping of `db_type` to actual column type used by database driver.
@@ -1047,6 +1057,13 @@ class QueryCompiler(object):
         elif isinstance(node, SQL):
             sql = node.value
             params = list(node.params)
+        elif isinstance(node, CompoundSelect):
+            l, lp = self.generate_select(
+                node.lhs, self._max_alias(alias_map), alias_map)
+            r, rp = self.generate_select(
+                node.rhs, self._max_alias(alias_map), alias_map)
+            sql = '%s %s %s' % (l, node.operator, r)
+            params = lp + rp
         elif isinstance(node, SelectQuery):
             max_alias = self._max_alias(alias_map)
             alias_copy = alias_map and alias_map.copy() or None
@@ -1160,15 +1177,18 @@ class QueryCompiler(object):
         alias_map = alias_map or {}
         alias_map.update(self.calculate_alias_map(query, start))
 
-        stmt = 'SELECT DISTINCT' if query._distinct else 'SELECT'
-        select_clause = Clause(*query._select)
-        select_clause.glue = ', '
-
-        clauses = [SQL(stmt), select_clause, SQL('FROM')]
-        if query._from is None:
-            clauses.append(model._as_entity().alias(alias_map[model]))
+        if isinstance(query, CompoundSelect):
+            clauses = [query]
         else:
-            clauses.append(CommaClause(*query._from))
+            stmt = 'SELECT DISTINCT' if query._distinct else 'SELECT'
+            select_clause = Clause(*query._select)
+            select_clause.glue = ', '
+
+            clauses = [SQL(stmt), select_clause, SQL('FROM')]
+            if query._from is None:
+                clauses.append(model._as_entity().alias(alias_map[model]))
+            else:
+                clauses.append(CommaClause(*query._from))
 
         join_clauses = self.generate_joins(query._joins, model, alias_map)
         if join_clauses:
@@ -1258,7 +1278,7 @@ class QueryCompiler(object):
         if field.on_delete:
             ddl.append(SQL('ON DELETE %s' % field.on_delete))
         if field.on_update:
-            ddl.append(SQL('ON UPDATE %s' % field.on_delete))
+            ddl.append(SQL('ON UPDATE %s' % field.on_update))
         return Clause(*ddl)
 
     def return_parsed_node(function_name):
@@ -1320,7 +1340,7 @@ class QueryCompiler(object):
             SQL(statement),
             Entity(index),
             SQL('ON'),
-            Entity(tbl_name),
+            model_class._as_entity(),
             EnclosedClause(*[field._as_entity() for field in fields]),
             *extra)
     create_index = return_parsed_node('_create_index')
@@ -1414,18 +1434,25 @@ class ExtQueryResultWrapper(QueryResultWrapper):
         for i in range(len(description)):
             func = identity
             column = description[i][0]
-            if column in model._meta.columns:
+            found = False
+            if self.column_meta is not None:
+                select_column = self.column_meta[i]
+                if isinstance(select_column, Field):
+                    func = select_column.python_value
+                    column = select_column._alias or select_column.name
+                    found = True
+                elif (isinstance(select_column, Func) and
+                        isinstance(select_column.arguments[0], Field)):
+                    if select_column._coerce:
+                        # Special-case handling aggregations.
+                        func = select_column.arguments[0].python_value
+                    found = True
+
+            if not found and column in model._meta.columns:
                 field_obj = model._meta.columns[column]
                 column = field_obj.name
                 func = field_obj.python_value
-            elif self.column_meta is not None:
-                select_column = self.column_meta[i]
-                # Special-case handling aggregations.
-                if (isinstance(select_column, Func) and
-                        isinstance(select_column.arguments[0], Field)):
-                    func = select_column.arguments[0].python_value
-                else:
-                    func = identity
+
             conv.append((i, column, func))
         self.conv = conv
 
@@ -1735,9 +1762,7 @@ class SelectQuery(Query):
     def __init__(self, model_class, *selection):
         super(SelectQuery, self).__init__(model_class)
         self.require_commit = self.database.commit_select
-        self._explicit_selection = len(selection) > 0
-        selection = selection or model_class._meta.get_fields()
-        self._select = self._model_shorthand(selection)
+        self.__select(*selection)
         self._from = None
         self._group_by = None
         self._having = None
@@ -1792,9 +1817,31 @@ class SelectQuery(Query):
                 accum.extend(arg._meta.get_fields())
         return accum
 
+    def compound_op(operator):
+        def inner(self, other):
+            return CompoundSelect(self.model_class, self, operator, other)
+        return inner
+    __or__ = compound_op('UNION')
+    __and__ = compound_op('INTERSECT')
+    __sub__ = compound_op('EXCEPT')
+
+    def __xor__(self, rhs):
+        # Symmetric difference, should just be (self | rhs) - (self & rhs)...
+        wrapped_rhs = self.model_class.select(SQL('*')).from_(
+            EnclosedClause((self & rhs)).alias('_')).order_by()
+        return (self | rhs) - wrapped_rhs
+
+    def __select(self, *selection):
+        self._explicit_selection = len(selection) > 0
+        selection = selection or self.model_class._meta.get_fields()
+        self._select = self._model_shorthand(selection)
+    select = returns_clone(__select)
+
     @returns_clone
     def from_(self, *args):
-        self._from = list(args)
+        self._from = None
+        if args:
+            self._from = list(args)
 
     @returns_clone
     def group_by(self, *args):
@@ -1916,10 +1963,13 @@ class SelectQuery(Query):
                 return False
         return True
 
+    def get_query_meta(self):
+        return (self._select, self._joins)
+
     def execute(self):
         if self._dirty or not self._qr:
             model_class = self.model_class
-            query_meta = [self._select, self._joins]
+            query_meta = self.get_query_meta()
             if self._tuples:
                 ResultWrapper = TuplesQueryResultWrapper
             elif self._dicts:
@@ -1952,6 +2002,24 @@ class SelectQuery(Query):
         res.fill_cache(index)
         return res._result_cache[value]
 
+class CompoundSelect(SelectQuery):
+    def __init__(self, model_class, lhs=None, operator=None, rhs=None):
+        self.lhs = lhs
+        self.operator = operator
+        self.rhs = rhs
+        super(CompoundSelect, self).__init__(model_class, [])
+
+    def _clone_attributes(self, query):
+        query = super(CompoundSelect, self)._clone_attributes(query)
+        query.lhs = self.lhs
+        query.operator = self.operator
+        query.rhs = self.rhs
+        return query
+
+    def get_query_meta(self):
+        return self.lhs.get_query_meta()
+
+
 class UpdateQuery(Query):
     def __init__(self, model_class, update=None):
         self._update = update
@@ -2039,6 +2107,7 @@ class ExceptionWrapper(object):
 class Database(object):
     commit_select = False
     compiler_class = QueryCompiler
+    drop_cascade = True
     field_overrides = {}
     foreign_keys = True
     for_update = False
@@ -2050,6 +2119,7 @@ class Database(object):
     savepoints = True
     sequences = False
     subquery_delete_same_table = True
+    window_functions = False
 
     exceptions = {
         'DatabaseError': DatabaseError,
@@ -2238,9 +2308,10 @@ class Database(object):
             qc = self.compiler()
             return self.execute_sql(*qc.create_sequence(seq))
 
-    def drop_table(self, model_class, fail_silently=False):
+    def drop_table(self, model_class, fail_silently=False, cascade=False):
         qc = self.compiler()
-        return self.execute_sql(*qc.drop_table(model_class, fail_silently))
+        return self.execute_sql(*qc.drop_table(
+            model_class, fail_silently, cascade))
 
     def drop_sequence(self, seq):
         if self.sequences:
@@ -2251,6 +2322,7 @@ class Database(object):
         return fn.EXTRACT(Clause(date_part, R('FROM'), date_field))
 
 class SqliteDatabase(Database):
+    drop_cascade = False
     foreign_keys = False
     limit_max = -1
     op_overrides = {
@@ -2295,6 +2367,7 @@ class PostgresqlDatabase(Database):
     interpolation = '%s'
     reserved_tables = ['user']
     sequences = True
+    window_functions = True
 
     register_unicode = True
 
@@ -2796,8 +2869,8 @@ class Model(with_metaclass(BaseModel)):
                 db.create_index(cls, fields, unique)
 
     @classmethod
-    def drop_table(cls, fail_silently=False):
-        cls._meta.database.drop_table(cls, fail_silently)
+    def drop_table(cls, fail_silently=False, cascade=False):
+        cls._meta.database.drop_table(cls, fail_silently, cascade)
 
     @classmethod
     def _as_entity(cls):
