@@ -15,18 +15,30 @@ from peewee import QueryCompiler
 from peewee import SelectQuery
 
 from psycopg2 import extensions
+from psycopg2.extensions import adapt
+from psycopg2.extensions import AsIs
+from psycopg2.extensions import register_adapter
 from psycopg2.extras import register_hstore
+try:
+    from psycopg2.extras import Json
+except:
+    Json = None
 
 
-class ObjectSlice(Node):
+class _LookupNode(Node):
     def __init__(self, node, parts):
         self.node = node
         self.parts = parts
-        super(ObjectSlice, self).__init__()
+        super(_LookupNode, self).__init__()
 
     def clone_base(self):
-        return ObjectSlice(self.node, list(self.parts))
+        return type(self)(self.node, list(self.parts))
 
+class JsonLookup(_LookupNode):
+    def __getitem__(self, value):
+        return JsonLookup(self.node, self.parts + [value])
+
+class ObjectSlice(_LookupNode):
     @classmethod
     def create(cls, node, value):
         if isinstance(value, slice):
@@ -40,38 +52,64 @@ class ObjectSlice(Node):
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
 
+class _Array(Node):
+    def __init__(self, field, items):
+        self.field = field
+        self.items = items
+        super(_Array, self).__init__()
 
-class ArrayField(Field):
-    def __init__(self, field_class=IntegerField, dimensions=1, *args,
-                 **kwargs):
-        kwargs['field_class'] = field_class
-        kwargs['dimensions'] = dimensions
+def adapt_array(arr):
+    items = adapt(arr.items).getquoted()
+    return AsIs(items + '::%s%s' % (
+        arr.field.get_column_type(),
+        '[]'* arr.field.dimensions))
+register_adapter(_Array, adapt_array)
+
+
+class IndexedField(Field):
+    def __init__(self, index_type='GiST', *args, **kwargs):
+        kwargs.setdefault('index', True)  # By default, use an index.
+        super(IndexedField, self).__init__(*args, **kwargs)
+        self.index_type = index_type
+
+
+class ArrayField(IndexedField):
+    def __init__(self, field_class=IntegerField, dimensions=1,
+                 index_type='GIN', *args, **kwargs):
         self.__field = field_class(*args, **kwargs)
+        self.dimensions = dimensions
         self.db_field = self.__field.get_db_field()
-        super(ArrayField, self).__init__(*args, **kwargs)
+        super(ArrayField, self).__init__(
+            index_type=index_type, *args, **kwargs)
 
-    def get_template(self):
-        brackets = ('[]' * self.attributes['dimensions'])
-        return self.__field.get_template() + brackets
-
-    def field_attributes(self):
-        return self.__field.field_attributes()
+    def __ddl_column__(self, column_type):
+        sql = self.__field.__ddl_column__(column_type)
+        sql.value += '[]' * self.dimensions
+        return sql
 
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
 
+    def contains(self, *items):
+        return Expression(self, OP_ACONTAINS, _Array(self, list(items)))
+
+    def contains_any(self, *items):
+        return Expression(self, OP_ACONTAINS_ANY, _Array(self, list(items)))
+
 
 class DateTimeTZField(DateTimeField):
     db_field = 'datetime_tz'
 
 
-class HStoreField(Field):
+class HStoreField(IndexedField):
     db_field = 'hash'
 
     def __init__(self, *args, **kwargs):
-        kwargs['index'] = True  # always use an Index
         super(HStoreField, self).__init__(*args, **kwargs)
 
+    def __getitem__(self, key):
+        return Expression(self, OP_HKEY, Param(key))
+
     def keys(self):
         return fn.akeys(self)
 
@@ -107,6 +145,21 @@ class HStoreField(Field):
         return Expression(self, OP_HCONTAINS_ANY_KEY, Param(value))
 
 
+class JSONField(Field):
+    db_field = 'json'
+
+    def __init__(self, *args, **kwargs):
+        if Json is None:
+            raise Exception('Your version of psycopg2 does not support JSON.')
+        super(JSONField, self).__init__(*args, **kwargs)
+
+    def db_value(self, value):
+        return Json(value)
+
+    def __getitem__(self, value):
+        return JsonLookup(self, [value])
+
+
 class UUIDField(Field):
     db_field = 'uuid'
 
@@ -117,32 +170,51 @@ class UUIDField(Field):
         return uuid.UUID(value)
 
 
+OP_HKEY = 'key'
 OP_HUPDATE = 'H@>'
 OP_HCONTAINS_DICT = 'H?&'
 OP_HCONTAINS_KEYS = 'H?'
 OP_HCONTAINS_KEY = 'H?|'
 OP_HCONTAINS_ANY_KEY = 'H||'
+OP_ACONTAINS = 'A@>'
+OP_ACONTAINS_ANY = 'A||'
 
 
 class PostgresqlExtCompiler(QueryCompiler):
-    def parse_create_index(self, model_class, fields, unique=False):
-        parts = super(PostgresqlExtCompiler, self).parse_create_index(
+    def _create_index(self, model_class, fields, unique=False):
+        clause = super(PostgresqlExtCompiler, self)._create_index(
             model_class, fields, unique)
-        # If this index is on an HStoreField, be sure to specify the
-        # GIST index immediately before the column names.
-        if any(map(lambda f: isinstance(f, HStoreField), fields)):
-            parts.insert(-1, 'USING GIST')
-        return parts
+        # Allow fields to specify a type of index.  HStore and Array fields
+        # may want to use GiST indexes, for example.
+        index_type = None
+        for field in fields:
+            if isinstance(field, IndexedField):
+                index_type = field.index_type
+        if index_type:
+            clause.nodes.insert(-1, SQL('USING %s' % index_type))
+        return clause
 
     def _parse(self, node, alias_map, conv):
         sql, params, unknown = super(PostgresqlExtCompiler, self)._parse(
             node, alias_map, conv)
-        if unknown and isinstance(node, ObjectSlice):
-            unknown = False
-            sql, params = self.parse_node(node.node)
-            # Postgresql uses 1-based indexes.
-            parts = [str(part + 1) for part in node.parts]
-            sql = '%s[%s]' % (sql, ':'.join(parts))
+        if unknown:
+            if isinstance(node, ObjectSlice):
+                unknown = False
+                sql, params = self.parse_node(node.node)
+                # Postgresql uses 1-based indexes.
+                parts = [str(part + 1) for part in node.parts]
+                sql = '%s[%s]' % (sql, ':'.join(parts))
+            if isinstance(node, JsonLookup):
+                unknown = False
+                sql, params = self.parse_node(node.node)
+                lookups = [sql]
+                for part in node.parts:
+                    part_sql, part_params = self.parse_node(part)
+                    lookups.append(part_sql)
+                    params.extend(part_params)
+                # The last lookup should be converted to text.
+                head, tail = lookups[:-1], lookups[-1]
+                sql = '->>'.join(('->'.join(head), tail))
         return sql, params, unknown
 
 
@@ -198,15 +270,20 @@ class ServerSideSelectQuery(SelectQuery):
 
 
 PostgresqlExtDatabase.register_fields({
-    'hash': 'hstore',
     'datetime_tz': 'timestamp with time zone',
-    'uuid': 'uuid'})
+    'hash': 'hstore',
+    'json': 'json',
+    'uuid': 'uuid',
+})
 PostgresqlExtDatabase.register_ops({
     OP_HCONTAINS_DICT: '@>',
     OP_HCONTAINS_KEYS: '?&',
     OP_HCONTAINS_KEY: '?',
     OP_HCONTAINS_ANY_KEY: '?|',
+    OP_HKEY: '->',
     OP_HUPDATE: '||',
+    OP_ACONTAINS: '@>',
+    OP_ACONTAINS_ANY: '&&',
 })
 
 def ServerSide(select_query):
